#![allow(clippy::println_empty_string)]
use std::{
    fs::File,
    io::{Read, Write},
    ops::Neg,
    u8,
};

// takes all charaters in bdf and returns a vec of each character row byte in order, normalized to
// width & height of the font (only works with 8 or fewer pixel wide fonts, should work for any height)
fn bdf_to_vec(path: &str) -> Vec<u8> {
    let mut bdf_vec = Vec::new();
    let file = File::open(path).unwrap();
    let mut bdf_font_bytes = Vec::new();
    for byte in file.bytes() {
        bdf_font_bytes.push(byte.unwrap());
    }
    let bdf_font = bdf_parser::BdfFont::parse(&bdf_font_bytes).unwrap();
    for glyph in bdf_font.glyphs.iter() {
        let glyph_offset_x = glyph.bounding_box.offset.x;
        let glyph_offset_y = glyph.bounding_box.offset.y;
        let glyph_height = glyph.bounding_box.size.y;
        let font_height = bdf_font.metadata.bounding_box.size.y;
        let font_offset_y = bdf_font.metadata.bounding_box.offset.y;

        let top_space = font_height + font_offset_y - glyph_height - glyph_offset_y;
        for _i in 0..top_space {
            bdf_vec.push(0x00);
        }
        for bitmap in glyph.bitmap.iter() {
            bdf_vec.push(bitmap >> glyph_offset_x);
        }

        let bottom_space = font_offset_y.neg() + glyph_offset_y;
        for _i in 0..bottom_space {
            bdf_vec.push(0x00);
        }
    }
    bdf_vec
}

// takes an vec of ordered characters and translates them for use with the character rom
// TODO: make this work for any arbitrary char rom pin format using some kinda interface
fn reorder_bitmap(bitmap: &[u8], font_height: usize) -> Vec<u8> {
    let mut rom = vec![0; 0x8000]; // create vec the size of character rom

    for row in 0..font_height {
        for ascii_address in 0..u8::MAX {
            // first 8 bits of address pick character
            // next 5 bits pick row, and TODO: final 2 pick character set
            let byte = bitmap[ascii_address as usize * font_height + row];
            let rom_index: u16 = ((row as u16) << 8) + ascii_address as u16;
            rom[rom_index as usize] = byte;
        }
    }
    rom
}

fn main() {
    let bits = bdf_to_vec("Cozette8-13.bdf");
    let mut rom = File::create("./cozette.rom").unwrap();
    let reordered = reorder_bitmap(&bits, 13);
    rom.write_all(&reordered).unwrap();
}
